require("geocomp")

local method = arg[1] or "minX"
local infileName = arg[2]
local outfileName = arg[3]

pointCloud = {}

function pointCloud.barycenter( points )
  local center = { x = 0 , y = 0 }
  for i = 1 , #points do
    center.x = center.x +  points[i].x
    center.y = center.y +  points[i].y
  end
  center.x = center.x/#points
  center.y = center.y/#points
  return center
end

function pointCloud.minX( points )
  local minX = {x = math.huge , y = math.huge } 
  for i = 1 , #points do
    if( minX.x > points[i].x ) then
      minX = points[i]
    end
  end
  return minX 
end

function pointCloud.minY( points )
  local minY = {x = math.huge , y = math.huge } 
  for i = 1 , #points do
    if( minY.y > points[i].y ) then
      minY = points[i] 
    end
  end
  return minY
end

function classifyAngle( p1 , p2 )
  vectorProduct = ((p1.x-PC.x)*(p2.y-PC.y))-((p1.y-PC.y)*(p2.x-PC.x));
  if( math.abs(vectorProduct) < 1e-12 ) then 
    local norm1 = ((p1.x-PC.x)*(p1.x-PC.x)+(p1.y-PC.y)*(p1.y-PC.y))
    local norm2 = ((p2.x-PC.x)*(p2.x-PC.x)+(p2.y-PC.y)*(p2.y-PC.y))
    return ( norm1 < norm2 ) 
  end
  return  ( vectorProduct < 0 )
end

--------------------------
-- Simple Polygon
--------------------------

local inFile = io.stdin
if( infileName ) then inFile = assert(io.open(infileName,"r")) end
local points = {}
local tp  
local data  
local bBoxTmp
local bBox

while true do
    tp , data , bBoxTmp = geocomp.fileGetObjc( inFile  )
    if( tp == nil ) then break end
    if( tp == "points" ) then
	points = data
        bBox = bBoxTmp
    end
end

if( points == nil ) then 
  io.stderr:write("-- No Points --") 
  return 
end

local bary = {}
local minX = {}

if( pointCloud[method]~= nil ) then 
  PC = pointCloud[method](points) 
else
  io.stderr:write( "-- Invalid Method! -- " )
  return
end


table.sort( points , classifyAngle )

-- writting the EPS file

local tableLine = {}

local dx = (bBox.Xmax - bBox.Xmin)*0.02
local dy = (bBox.Ymax - bBox.Ymin)*0.02
local dataWindow = { Xmin = bBox.Xmin - dx , Ymin = bBox.Ymin - dy , Xmax = bBox.Xmax + dx , Ymax = bBox.Ymax + dy } 

geocomp.epsPointRadius = 2
geocomp.epsSetHead( dataWindow )
local epsHead = geocomp.epsDohead( {} , bBox , tableLine )
geocomp.epsDoscale( geocomp.epsScaleConst , tableLine )
geocomp.epsDotranslate( {x = -dataWindow.Xmin , y = -dataWindow.Ymin} , tableLine )
geocomp.epsDoSetlinewidth( geocomp.epsLinewidth , tableLine )

geocomp.epsDopolygon( points , tableLine )
geocomp.epsDopoints( points , tableLine )
geocomp.epsDotail( tableLine )

local outFile = io.stdout
if( outfileName ) then outFile = assert( io.open(outfileName,"w") ) end
for i = 1 , #tableLine do
  outFile:write( tableLine[i] )
end

if( outfileName ) then print( "File generated " .. outfileName .. " --- using method = " .. method ) end
